# Copyright 2014, Microsoft Corporation. All rights reserved.
# You may use this code and information and create derivative works of it,
# provided that the following conditions are met:
# 1. This code and information and any derivative works may only be used for
# troubleshooting a) Windows and b) products for Windows, in either case using
# the Windows Troubleshooting Platform
# 2. Any copies of this code and information
# and any derivative works must retain the above copyright notice, this list of
# conditions and the following disclaimer.
# 3. THIS CODE AND INFORMATION IS PROVIDED ``AS IS'' WITHOUT WARRANTY OF ANY KIND,
# WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. IF THIS CODE AND
# INFORMATION IS USED OR MODIFIED, THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION
# WITH THE USE OF THIS CODE AND INFORMATION REMAINS WITH THE USER.
# AutomatedFixUnattended.ps1

PARAM(
$AutomatedFixPackage
)

Function Print-Help
{
    $automatedFixVersion = "1.0.0.0"
    $helpScript = @"

TOPIC 
   Microsoft (R) Automated Fix unattended installation Script Help System - Version $automatedFixVersion
   Copyright (c) Microsoft Corporation. All rights reserved.

DESCRIPTION:
   Silent install one Automated Fix MiniPack solution.

COMMAND HELP:
   AutomatedFixUnattended.ps1 -AutomatedFixPackage [XXXXX.mini.diagcab]
"@
    Write-Host $helpScript
}

Function Write-AutomatedFixExecutionLog {
    Param(
    [string]$Message,
    [string]$Path,
    [string]$Level = "info"
    )

    if (-not $(Test-Path $path))
    {
        New-Item $path -Type File -ErrorAction SilentlyContinue >> null
    }
    
    if(Test-Path $path)
    {
        Switch -wildcard ($level) {
            'info'
            {
                Write-Output "[$level] : $(Get-Date) ---- $Message" | Out-File -FilePath $Path -Append
                Write-Host -ForegroundColor Green $Message
            }
            'warning'
            {
                Write-Output "[$level] : $(Get-Date) ---- $Message" | Out-File -FilePath $Path -Append
                Write-Host -ForegroundColor Yellow $Message
            }
            'error'
            {
                Write-Output "[$level] : $(Get-Date) ---- $Message" | Out-File -FilePath $Path -Append
                Write-Host -ForegroundColor Red $Message
            }
            default {}
        }
    }
}

$AutomatedFixExecutionLog = Join-Path $env:tmp $("MicrosoftAutomatedFix_" + [System.DateTime]::Now.ToString("yyyyMMddHHmmss") + ".log")
$architecture = -1
$proc = @(Get-WmiObject Win32_Processor -ErrorAction SilentlyContinue)
if ($proc -ne $null -and $proc.Count -gt 0)
{
    $architecture = $proc[0].Architecture
}

trap [Exception]
{
    if ($architecture -eq 5)
    {
        Write-AutomatedFixExecutionLog -Path $AutomatedFixExecutionLog -Level "error" `
            -Message "The AutomatedFixUnattended script has limited support on Windows RT and experienced errors."
    }
    Write-AutomatedFixExecutionLog -Path $AutomatedFixExecutionLog -Level "error" `
        -Message $("The AutomatedFixUnattended script fails with unexpected errors: `r`n" + $Error[0].ToString())
    #PAUSE
    return
}

if ([System.Environment]::OSVersion.Version -lt [System.Version]"6.1.7600.0")
{
    Write-AutomatedFixExecutionLog -Path $AutomatedFixExecutionLog -Level "warning" `
        -Message "The AutomatedFixUnattended script can only work on the Windows 7 or the later version Windows operating systems"
    return
}

if($PsBoundParameters.Count -ne 1 -or $args.Count -gt 0)
{
    Write-AutomatedFixExecutionLog -Path $AutomatedFixExecutionLog -Level "error" `
        -Message "The parameters of this script is not correct.  See the following Command Help for the details."
    Print-Help
    return
}

if($AutomatedFixPackage -eq $null -or (-not $(Test-Path $AutomatedFixPackage)))
{
    Write-AutomatedFixExecutionLog -Path $AutomatedFixExecutionLog -Level "error" `
        -Message "The Automated Fix Package file path is not correct or the file is not exist."
    Print-Help
    return
}

if ((-not $AutomatedFixPackage.EndsWith(".mini.diagcab")) -or ($AutomatedFixPackage.Length -le 13))
{
    Write-AutomatedFixExecutionLog -Path $AutomatedFixExecutionLog -Level "error" `
        -Message "The script only help you silently use the Automated Fix solutions with the '.mini.diagcab' extension."
    Print-Help
    #PAUSE
    return
}

$automatedFixFileNameWithOutExtension = $automatedFixPackage.Substring(0, $automatedFixPackage.Length - 13)

$tempFolder = [System.Guid]::NewGuid().ToString() -replace "-",""
$tempPath = Join-Path $(Join-Path $env:tmp "AutomatedFixTempExpension") $tempFolder
md $tempPath >> null
expand $AutomatedFixPackage -F:* $tempPath >> null
$automatedFixFolderPath = Join-Path $tempPath $automatedFixFileNameWithOutExtension

if (Test-Path $automatedFixFolderPath)
{
    Import-Module TroubleshootingPack
    $automatedFixPack = Get-TroubleshootingPack -Path $automatedFixFolderPath
    if ($automatedFixPack.RequiresElevation -eq $true -and $architecture -ne 5)
    {
        $identity = [System.Security.Principal.WindowsIdentity]::GetCurrent()
        $principal = New-Object System.Security.Principal.WindowsPrincipal -ArgumentList $identity
        if (-not $($principal.IsInRole([System.Security.Principal.WindowsBuiltInRole]::Administrator)))
        {
            Write-AutomatedFixExecutionLog -Path $AutomatedFixExecutionLog -Level "error" `
                -Message "The Automated Fix solution requires to run in the elevated Administrator account."
            Del -Recurse $tempPath -Force -ErrorAction SilentlyContinue
            return
        }
    }	
    Invoke-TroubleshootingPack -Pack $automatedFixPack -Unattended
}

Del -Recurse $tempPath -Force -ErrorAction SilentlyContinue

Write-AutomatedFixExecutionLog -Path $AutomatedFixExecutionLog -Level "info" `
    -Message "The Automated Fix solution applies successfully."